from typing import Dict, Optional

import requests
from huggingface_hub import HfFolder
from loguru import logger

from autotrain import config
from autotrain.tasks import TASKS


FORMAT_TAG = "\033[{code}m"
RESET_TAG = FORMAT_TAG.format(code=0)
BOLD_TAG = FORMAT_TAG.format(code=1)
RED_TAG = FORMAT_TAG.format(code=91)
GREEN_TAG = FORMAT_TAG.format(code=92)
YELLOW_TAG = FORMAT_TAG.format(code=93)
PURPLE_TAG = FORMAT_TAG.format(code=95)
CYAN_TAG = FORMAT_TAG.format(code=96)


class UnauthenticatedError(Exception):
    pass


class UnreachableAPIError(Exception):
    pass


def get_auth_headers(token: str, prefix: str = "Bearer"):
    return {"Authorization": f"{prefix} {token}"}


def http_get(
    path: str,
    token: str,
    domain: str = config.AUTOTRAIN_BACKEND_API,
    token_prefix: str = "Bearer",
    suppress_logs: bool = False,
    **kwargs,
) -> requests.Response:
    """HTTP GET request to the AutoNLP API, raises UnreachableAPIError if the API cannot be reached"""
    try:
        response = requests.get(
            url=domain + path, headers=get_auth_headers(token=token, prefix=token_prefix), **kwargs
        )
    except requests.exceptions.ConnectionError:
        raise UnreachableAPIError("❌ Failed to reach AutoNLP API, check your internet connection")
    response.raise_for_status()
    return response


def http_post(
    path: str,
    token: str,
    payload: Optional[Dict] = None,
    domain: str = config.AUTOTRAIN_BACKEND_API,
    suppress_logs: bool = False,
    **kwargs,
) -> requests.Response:
    """HTTP POST request to the AutoNLP API, raises UnreachableAPIError if the API cannot be reached"""
    try:
        response = requests.post(
            url=domain + path, json=payload, headers=get_auth_headers(token=token), allow_redirects=True, **kwargs
        )
    except requests.exceptions.ConnectionError:
        raise UnreachableAPIError("❌ Failed to reach AutoNLP API, check your internet connection")
    response.raise_for_status()
    return response


def get_task(task_id: int) -> str:
    for key, value in TASKS.items():
        if value == task_id:
            return key
    return "❌ Unsupported task! Please update autonlp"


def get_user_token():
    return HfFolder.get_token()


def user_authentication(token):
    headers = {}
    cookies = {}
    if token.startswith("hf_"):
        headers["Authorization"] = f"Bearer {token}"
    else:
        cookies = {"token": token}
    try:
        response = requests.get(
            config.HF_API + "/api/whoami-v2",
            headers=headers,
            cookies=cookies,
            timeout=3,
        )
    except (requests.Timeout, ConnectionError) as err:
        logger.error(f"Failed to request whoami-v2 - {repr(err)}")
        raise Exception("Hugging Face Hub is unreachable, please try again later.")
    return response.json()


def get_project_cost(username, token, task, num_samples, num_models):
    task_id = TASKS[task]
    pricing = http_get(
        path=f"/pricing/compute?username={username}&task_id={task_id}&num_samples={num_samples}&num_models={num_models}",
        token=token,
    )
    return pricing.json()["price"]



from dataclasses import dataclass
from typing import List, Optional

import pandas as pd
from loguru import logger

from autotrain.preprocessor.dreambooth import DreamboothPreprocessor
from autotrain.preprocessor.tabular import (
    TabularBinaryClassificationPreprocessor,
    TabularMultiClassClassificationPreprocessor,
    TabularSingleColumnRegressionPreprocessor,
)
from autotrain.preprocessor.text import (
    TextBinaryClassificationPreprocessor,
    TextMultiClassClassificationPreprocessor,
    TextSingleColumnRegressionPreprocessor,
)


@dataclass
class AutoTrainDreamboothDataset:
    num_concepts: int
    concept_images: List[List[str]]
    concept_names: List[str]
    token: str
    project_name: str
    username: str

    def __str__(self) -> str:
        info = f"Dataset: {self.project_name} ({self.task})\n"
        return info

    def __post_init__(self):
        self.task = "dreambooth"
        logger.info(self.__str__())

    @property
    def num_samples(self):
        return sum([len(concept) for concept in self.concept_images])

    def prepare(self):
        preprocessor = DreamboothPreprocessor(
            num_concepts=self.num_concepts,
            concept_images=self.concept_images,
            concept_names=self.concept_names,
            token=self.token,
            project_name=self.project_name,
            username=self.username,
        )
        preprocessor.prepare()


@dataclass
class AutoTrainDataset:
    train_data: str
    task: str
    token: str
    project_name: str
    username: str
    column_mapping: Optional[str] = None
    valid_data: Optional[str] = None
    percent_valid: Optional[float] = None

    def __str__(self) -> str:
        info = f"Dataset: {self.project_name} ({self.task})\n"
        info += f"Train data: {self.train_data}\n"
        info += f"Valid data: {self.valid_data}\n"
        info += f"Column mapping: {self.column_mapping}\n"
        return info

    def __post_init__(self):
        if not self.valid_data and self.percent_valid is None:
            self.percent_valid = 0.2
        elif self.valid_data and self.percent_valid is not None:
            raise ValueError("You can only specify one of valid_data or percent_valid")
        elif self.valid_data:
            self.percent_valid = 0.0
        logger.info(self.__str__())

    @property
    def num_samples(self):
        train_df = []
        for file in self.train_data:
            if isinstance(file, pd.DataFrame):
                train_df.append(file)
            else:
                train_df.append(pd.read_csv(file))
        if len(train_df) > 1:
            train_df = pd.concat(train_df)
        else:
            train_df = train_df[0]

        valid_df = None
        if len(self.valid_data) > 0:
            valid_df = []
            for file in self.valid_data:
                if isinstance(file, pd.DataFrame):
                    valid_df.append(file)
                else:
                    valid_df.append(pd.read_csv(file))
            if len(valid_df) > 1:
                valid_df = pd.concat(valid_df)
            else:
                valid_df = valid_df[0]

        logger.info(train_df.head())
        if valid_df is not None:
            logger.info(valid_df.head())

        return len(train_df) + len(valid_df) if valid_df is not None else len(train_df)

    def prepare(self):
        logger.info(self.train_data)
        train_df = []
        for file in self.train_data:
            if isinstance(file, pd.DataFrame):
                train_df.append(file)
            else:
                train_df.append(pd.read_csv(file))
        if len(train_df) > 1:
            train_df = pd.concat(train_df)
        else:
            train_df = train_df[0]

        valid_df = None
        if len(self.valid_data) > 0:
            valid_df = []
            for file in self.valid_data:
                if isinstance(file, pd.DataFrame):
                    valid_df.append(file)
                else:
                    valid_df.append(pd.read_csv(file))
            if len(valid_df) > 1:
                valid_df = pd.concat(valid_df)
            else:
                valid_df = valid_df[0]

        logger.info(train_df.head())
        if valid_df is not None:
            logger.info(valid_df.head())

        if self.task == "text_binary_classification":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextBinaryClassificationPreprocessor(
                train_data=train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()

        elif self.task == "text_multi_class_classification":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextMultiClassClassificationPreprocessor(
                train_data=train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()

        elif self.task == "text_single_column_regression":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextSingleColumnRegressionPreprocessor(
                train_data=train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_binary_classification":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularBinaryClassificationPreprocessor(
                train_data=train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_multi_class_classification":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularMultiClassClassificationPreprocessor(
                train_data=train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_single_column_regression":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularSingleColumnRegressionPreprocessor(
                train_data=train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        else:
            raise ValueError(f"Task {self.task} not supported")



import os
import subprocess
from argparse import ArgumentParser

from . import BaseAutoTrainCommand


def run_app_command_factory(args):
    return RunAutoTrainAppCommand(
        args.port,
        args.host,
        args.task,
    )


class RunAutoTrainAppCommand(BaseAutoTrainCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        run_app_parser = parser.add_parser(
            "app",
            description="✨ Run AutoTrain app",
        )
        run_app_parser.add_argument(
            "--port",
            type=int,
            default=9000,
            help="Port to run the app on",
            required=False,
        )
        run_app_parser.add_argument(
            "--host",
            type=str,
            default="127.0.0.1",
            help="Host to run the app on",
            required=False,
        )
        run_app_parser.add_argument(
            "--task",
            type=str,
            required=False,
            help="Task to run",
        )
        run_app_parser.set_defaults(func=run_app_command_factory)

    def __init__(self, port, host, task):
        self.port = port
        self.host = host
        self.task = task

    def run(self):
        dirname = os.path.dirname(__file__)
        filename = os.path.join(dirname, "..", "app.py")
        cmd = [
            "streamlit",
            "run",
            filename,
            "--browser.gatherUsageStats",
            "false",
            "--browser.serverAddress",
            self.host,
            "--server.port",
            str(self.port),
            "--theme.base",
            "light",
        ]
        if "SPACE_ID" in os.environ:
            cmd.extend(["--server.enableXsrfProtection", "false"])
            cmd.extend(["--server.headless", "true"])
            cmd.extend(["--server.enableCORS", "false"])
            cmd.extend(["--server.fileWatcherType", "none"])

        if self.task:
            cmd.extend(["--", "--task", self.task])

        proc = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            shell=False,
            universal_newlines=True,
            bufsize=1,
        )
        with proc as p:
            try:
                for line in p.stdout:
                    print(line, end="")
            except KeyboardInterrupt:
                print("Killing app")
                p.kill()
                p.wait()
                raise



"""
Copyright 2023 The HuggingFace Team
"""

import time
from dataclasses import dataclass
from typing import Dict, List, Optional, Union

from loguru import logger

from autotrain.dataset import AutoTrainDataset, AutoTrainDreamboothDataset
from autotrain.languages import SUPPORTED_LANGUAGES
from autotrain.tasks import TASKS
from autotrain.utils import http_get, http_post


@dataclass
class Project:
    dataset: Union[AutoTrainDataset, AutoTrainDreamboothDataset]
    hub_model: Optional[str] = None
    job_params: Optional[List[Dict]] = None

    def __post_init__(self):
        self.token = self.dataset.token
        self.name = self.dataset.project_name
        self.username = self.dataset.username
        self.task = self.dataset.task

        if self.token is None:
            raise ValueError("❌ Please login using `huggingface-cli login`")

        if self.hub_model is not None and len(self.job_params) == 0:
            raise ValueError("❌ Job parameters are required when hub model is specified.")

        if self.hub_model is None and len(self.job_params) > 1:
            raise ValueError("❌ Only one job parameter is allowed in AutoTrain mode.")

        if len(self.job_params) == 1 and self.hub_model is None:
            if "source_language" in self.job_params[0] and "target_language" not in self.job_params[0]:
                self.language = self.job_params[0]["source_language"]
                # remove source language from job params
                self.job_params[0].pop("source_language")
            elif "source_language" in self.job_params[0] and "target_language" in self.job_params[0]:
                self.language = f'{self.job_params[0]["target_language"]}2{self.job_params[0]["source_language"]}'
                # remove source and target language from job params
                self.job_params[0].pop("source_language")
                self.job_params[0].pop("target_language")
            else:
                self.language = "unk"

            if "num_models" in self.job_params[0]:
                self.max_models = self.job_params[0]["num_models"]
                self.job_params[0].pop("num_models")
            elif "num_models" not in self.job_params[0] and "source_language" in self.job_params[0]:
                raise ValueError("❌ Please specify num_models in job_params when using AutoTrain model")
        else:
            self.language = "unk"
            self.max_models = len(self.job_params)

    def create(self):
        """Create a project and return it"""
        task_id = TASKS.get(self.task)
        if task_id is None:
            raise ValueError(f"❌ Invalid task selected. Please choose one of {TASKS.keys()}")
        language = str(self.language).strip().lower()
        if task_id is None:
            raise ValueError(f"❌ Invalid task specified. Please choose one of {list(TASKS.keys())}")

        if self.hub_model is not None:
            language = "unk"

        if language not in SUPPORTED_LANGUAGES:
            raise ValueError("❌ Invalid language. Please check supported languages in AutoTrain documentation.")

        payload = {
            "username": self.username,
            "proj_name": self.name,
            "task": task_id,
            "config": {
                "advanced": True,
                "autotrain": True
                if self.hub_model is None
                else False,  # TODO: autotrain should work with hub_model too. beacause, why not!
                "language": language,
                "max_models": self.max_models,
                "hub_model": self.hub_model,
                "params": self.job_params,
            },
        }
        logger.info(payload)
        json_resp = http_post(path="/projects/create", payload=payload, token=self.token).json()
        proj_name = json_resp["proj_name"]
        proj_id = json_resp["id"]
        created = json_resp["created"]

        if created is True:
            return proj_id
        raise ValueError(f"❌ Project with name {proj_name} already exists.")

    def approve(self, project_id):

        # Process data
        _ = http_post(
            path=f"/projects/{project_id}/data/start_processing",
            token=self.token,
        ).json()

        logger.info("⏳ Waiting for data processing to complete ...")
        is_data_processing_success = False
        while is_data_processing_success is not True:
            project_status = http_get(
                path=f"/projects/{project_id}",
                token=self.token,
            ).json()
            # See database.database.enums.ProjectStatus for definitions of `status`
            if project_status["status"] == 3:
                is_data_processing_success = True
                logger.info("✅ Data processing complete!")
            time.sleep(3)
        # Approve training job
        _ = http_post(
            path=f"/projects/{project_id}/start_training",
            token=self.token,
        ).json()



from dataclasses import dataclass

from autotrain.languages import SUPPORTED_LANGUAGES
from autotrain.tasks import TASKS


class LearningRate:
    TYPE = "float"
    MIN_VALUE = 1e-7
    MAX_VALUE = 1e-1
    DEFAULT = 1e-3
    STEP = 1e-6
    FORMAT = "%.2E"
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Learning Rate"


class Optimizer:
    TYPE = "str"
    DEFAULT = "adam"
    CHOICES = ["adam", "sgd"]
    STREAMLIT_INPUT = "selectbox"
    PRETTY_NAME = "Optimizer"


class Scheduler:
    TYPE = "str"
    DEFAULT = "linear"
    CHOICES = ["linear", "cosine"]
    STREAMLIT_INPUT = "selectbox"
    PRETTY_NAME = "Scheduler"


class TrainBatchSize:
    TYPE = "int"
    MIN_VALUE = 1
    MAX_VALUE = 128
    DEFAULT = 8
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Train Batch Size"


class Epochs:
    TYPE = "int"
    MIN_VALUE = 1
    MAX_VALUE = 1000
    DEFAULT = 10
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Epochs"


class PercentageWarmup:
    TYPE = "float"
    MIN_VALUE = 0.0
    MAX_VALUE = 1.0
    DEFAULT = 0.1
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Percentage Warmup"


class GradientAccumulationSteps:
    TYPE = "int"
    MIN_VALUE = 1
    MAX_VALUE = 100
    DEFAULT = 1
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Gradient Accumulation Steps"


class WeightDecay:
    TYPE = "float"
    MIN_VALUE = 0.0
    MAX_VALUE = 1.0
    DEFAULT = 0.0
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Weight Decay"


class SourceLanguage:
    TYPE = "str"
    DEFAULT = "en"
    CHOICES = SUPPORTED_LANGUAGES
    STREAMLIT_INPUT = "selectbox"
    PRETTY_NAME = "Source Language"


class TargetLanguage:
    TYPE = "str"
    DEFAULT = "en"
    CHOICES = SUPPORTED_LANGUAGES
    STREAMLIT_INPUT = "selectbox"
    PRETTY_NAME = "Target Language"


class NumModels:
    TYPE = "int"
    MIN_VALUE = 1
    MAX_VALUE = 25
    DEFAULT = 5
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Number of Models"


class DBNumSteps:
    TYPE = "int"
    MIN_VALUE = 100
    MAX_VALUE = 10000
    DEFAULT = 1500
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Number of Steps"


class DBTextEncoderStepsPercentage:
    TYPE = "int"
    MIN_VALUE = 1
    MAX_VALUE = 100
    DEFAULT = 30
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Text encoder steps percentage"


class DBPriorPreservation:
    TYPE = "bool"
    DEFAULT = False
    STREAMLIT_INPUT = "checkbox"
    PRETTY_NAME = "Prior preservation"


class ImageSize:
    TYPE = "int"
    MIN_VALUE = 64
    MAX_VALUE = 2048
    DEFAULT = 512
    STREAMLIT_INPUT = "number_input"
    PRETTY_NAME = "Image Size"


@dataclass
class Params:
    task: str
    training_type: str

    def __post_init__(self):
        # task should be one of the keys in TASKS
        if self.task not in TASKS:
            raise ValueError(f"task must be one of {TASKS.keys()}")
        self.task_id = TASKS[self.task]

        if self.training_type not in ("autotrain", "hub_model"):
            raise ValueError("training_type must be either autotrain or hub_model")

    def _dreambooth(self):
        return {
            "image_size": ImageSize,
            "learning_rate": LearningRate,
            "train_batch_size": TrainBatchSize,
            "num_steps": DBNumSteps,
            "text_encoder_steps_percentage": DBTextEncoderStepsPercentage,
            "prior_preservation": DBPriorPreservation,
        }

    def _tabular_binary_classification(self):
        return {
            "num_models": NumModels,
        }

    def _tabular_multi_class_classification(self):
        return self._tabular_binary_classification()

    def _tabular_single_column_regression(self):
        return self._tabular_binary_classification()

    def tabular_multi_label_classification(self):
        return self._tabular_binary_classification()

    def _text_binary_classification(self):
        if self.training_type == "hub_model":
            return {
                "learning_rate": LearningRate,
                "optimizer": Optimizer,
                "scheduler": Scheduler,
                "train_batch_size": TrainBatchSize,
                "epochs": Epochs,
                "percentage_warmup": PercentageWarmup,
                "gradient_accumulation_steps": GradientAccumulationSteps,
                "weight_decay": WeightDecay,
            }
        return {
            "source_language": SourceLanguage,
            "num_models": NumModels,
        }

    def _text_multi_class_classification(self):
        return self._text_binary_classification()

    def _text_entity_extraction(self):
        return self._text_binary_classification()

    def _text_single_column_regression(self):
        return self._text_binary_classification()

    def _text_natural_language_inference(self):
        return self._text_binary_classification()

    def get(self):
        if self.task == "text_binary_classification":
            return self._text_binary_classification()

        if self.task == "text_multi_class_classification":
            return self._text_multi_class_classification()

        if self.task == "text_entity_extraction":
            return self._text_entity_extraction()

        if self.task == "text_single_column_regression":
            return self._text_single_column_regression()

        if self.task == "text_natural_language_inference":
            return self._text_natural_language_inference()

        if self.task == "tabular_binary_classification":
            return self._tabular_binary_classification()

        if self.task == "tabular_multi_class_classification":
            return self._tabular_multi_class_classification()

        if self.task == "tabular_single_column_regression":
            return self._tabular_single_column_regression()

        if self.task == "tabular_multi_label_classification":
            return self.tabular_multi_label_classification()

        if self.task == "dreambooth":
            return self._dreambooth()

        raise ValueError(f"task {self.task} not supported")



import argparse
import copy
import os
import re

import pandas as pd
import streamlit as st
from huggingface_hub import HfApi
from huggingface_hub.utils import RepositoryNotFoundError
from loguru import logger
from st_aggrid import AgGrid, AgGridTheme, ColumnsAutoSizeMode, GridOptionsBuilder, GridUpdateMode

from autotrain.dataset import AutoTrainDataset, AutoTrainDreamboothDataset
from autotrain.params import Params
from autotrain.project import Project
from autotrain.tasks import COLUMN_MAPPING, NLP_TASKS, TABULAR_TASKS, TASK_TYPE_MAPPING, VISION_TASKS
from autotrain.utils import get_project_cost, get_user_token, user_authentication


def parse_args():
    """
    Parse command line arguments
    :return: parsed arguments
    """
    parser = argparse.ArgumentParser(description="AutoTrain app")
    parser.add_argument(
        "--task",
        type=str,
        required=False,
    )
    return parser.parse_args()


def does_repo_exist(repo_id, repo_type) -> bool:
    try:
        HfApi().repo_info(repo_id=repo_id, repo_type=repo_type)
        return True
    except RepositoryNotFoundError:
        return False


def verify_project_name(project_name, username, user_token):
    """
    Verify that the project name is valid
    :param project_name: name of the project
    :param username: username of the user
    :return: True if project name is valid, False otherwise
    """
    if project_name == "":
        st.error("Project name cannot be empty")
        return False
    if len(project_name) > 50:
        st.error("Project name cannot be longer than 50 characters")
        return False
    pattern = "^[A-Za-z0-9-]*$"
    if not re.match(pattern, project_name):
        st.error("Project name can only contain letters, numbers and hyphens")
        return False
    if user_token is None:
        st.error("You need to be logged in to create a project. Please login using `huggingface-cli login`")
        return False
    data_repo_name = f"{username}/{project_name}"
    if does_repo_exist(data_repo_name, "dataset"):
        st.error("A project with this name already exists")
        return False
    return True


def get_job_params(job_params, selected_rows, task, model_choice):
    """
    Get job parameters list of dicts for AutoTrain and HuggingFace Hub models
    :param job_params: job parameters
    :param selected_rows: selected rows
    :param task: task
    :param model_choice: model choice
    :return: job parameters list of dicts
    """
    if model_choice == "AutoTrain":
        if len(job_params) > 1:
            raise ValueError("❌ Only one job parameter is allowed for AutoTrain.")
        job_params[0].update({"task": task})
    elif model_choice == "HuggingFace Hub":
        for i in range(len(job_params)):
            job_params[i].update({"task": task})
        job_params = [job_params[i] for i in selected_rows]
    logger.info("***")
    logger.info(job_params)
    return job_params


def create_grid(jobs):
    """
    Create Aggrid for job parameters
    :param jobs: job parameters from streamlit session_state
    :return: Aggrid
    """

    df = pd.DataFrame(jobs)
    gb = GridOptionsBuilder.from_dataframe(df)
    gb.configure_default_column(
        cellStyle={"color": "black", "font-size": "12px"},
        suppressMenu=True,
        wrapHeaderText=True,
        autoHeaderHeight=True,
    )
    gb.configure_selection(
        selection_mode="multiple",
        use_checkbox=True,
        pre_selected_rows=list(range(len(df))),
        header_checkbox=True,
    )
    custom_css = {
        ".ag-header-cell-text": {"font-size": "12px", "text-overflow": "revert;", "font-weight": 700},
        # ".ag-theme-streamlit": {"transform": "scale(0.8)", "transform-origin": "0 0"},
    }
    gridOptions = gb.build()
    ag_resp = AgGrid(
        df,
        gridOptions=gridOptions,
        custom_css=custom_css,
        # allow_unsafe_jscode=True,
        columns_auto_size_mode=ColumnsAutoSizeMode.FIT_CONTENTS,
        theme=AgGridTheme.STREAMLIT,  # Only choices: AgGridTheme.STREAMLIT, AgGridTheme.ALPINE, AgGridTheme.BALHAM, AgGridTheme.MATERIAL
        # width='100%',
        update_mode=GridUpdateMode.MODEL_CHANGED,
        update_on="MANUAL",
        reload_data=False,
    )
    return ag_resp


def app():  # username, valid_orgs):
    st.sidebar.markdown(
        "<p style='text-align: center; font-size: 20px; font-weight: bold;'>AutoTrain Advanced</p>",
        unsafe_allow_html=True,
    )
    user_token = os.environ.get("HF_TOKEN", "")
    if len(user_token) == 0:
        user_token = get_user_token()
    if user_token is None:
        st.error(
            """You need to be logged in to create a project. Please login using `huggingface-cli login` or pass your HF token in an environment variable called `HF_TOKEN`
            \nIf you are running this app in Hugging Face Space, please clone it to your own user and add `HF_TOKEN` as a secret."""
        )
        return
    if len(user_token) == 0:
        return
    user_info = user_authentication(token=user_token)
    username = user_info["name"]
    user_can_pay = user_info["canPay"]

    orgs = user_info["orgs"]
    valid_orgs = [org for org in orgs if org["canPay"] is True]
    valid_orgs = [org for org in valid_orgs if org["roleInOrg"] in ("admin", "write")]
    valid_orgs = [org["name"] for org in valid_orgs]

    if user_can_pay is False and len(valid_orgs) == 0:
        st.error(
            "Please attach a CC to your account / join an organization with a CC attached to it to create a project"
        )
        return

    who_is_training = [username] + valid_orgs

    col1, col2 = st.columns(2)
    with col1:
        autotrain_username = st.selectbox("Who is training?", who_is_training)
    with col2:
        project_name = st.text_input("Project name", "my-project")

    if "task" in st.session_state:
        project_type = TASK_TYPE_MAPPING[st.session_state.task]
        task = st.session_state.task
    else:
        col1, col2 = st.columns(2)
        with col1:
            project_type = st.selectbox(
                "Project Type",
                [
                    "Natural Language Processing",
                    "Computer Vision",
                    "Tabular",
                ],
            )
        with col2:
            if project_type == "Natural Language Processing":
                task = st.selectbox("Task", list(NLP_TASKS.keys()))
            elif project_type == "Computer Vision":
                task = st.selectbox("Task", list(VISION_TASKS.keys()))
            elif project_type == "Tabular":
                task = st.selectbox("Task", list(TABULAR_TASKS.keys()))

    if task == "dreambooth":
        number_of_concepts = st.number_input("Number of concepts", min_value=1, max_value=5, value=1)
        tabs = st.tabs([f"Concept {i + 1}" for i in range(number_of_concepts)])
        for i in range(number_of_concepts):
            with tabs[i]:
                st.text_input(f"Concept {i + 1} token", key=f"dreambooth_concept_name_{i + 1}")
                st.file_uploader(
                    f"Concept {i + 1} images",
                    key=f"dreambooth_concept_images_{i + 1}",
                    type=["png", "jpg", "jpeg"],
                    accept_multiple_files=True,
                )
        training_data = "dreambooth"
    else:
        tab1, tab2 = st.tabs(["Training", "Validation"])
        with tab1:
            training_data = st.file_uploader(
                "Training Data" if project_type != "Computer Vision" else "Training Labels",
                type=["csv", "jsonl"] if project_type != "Computer Vision" else ["csv"],
                accept_multiple_files=True if project_type != "Computer Vision" else False,
            )
            if project_type == "Computer Vision":
                training_images = st.file_uploader("Training Images", type=["zip"])
        with tab2:
            validation_data = st.file_uploader(
                "Validation Data" if project_type != "Computer Vision" else "Validation Labels",
                type=["csv", "jsonl"] if project_type != "Computer Vision" else ["csv"],
                accept_multiple_files=True if project_type != "Computer Vision" else False,
            )
            if project_type == "Computer Vision":
                validation_images = st.file_uploader("Validation Images", type=["zip"])

    if "training_data" in locals() and training_data:
        if task != "dreambooth":
            st.markdown("###### Column mapping")
            # read column names
            temp_train_data = copy.deepcopy(training_data[0])
            if temp_train_data.name.endswith(".csv"):
                df = pd.read_csv(temp_train_data, nrows=0)
            elif temp_train_data.name.endswith(".jsonl"):
                df = pd.read_json(temp_train_data, lines=True, nrows=0)
            else:
                raise ValueError("Unknown file type")
            columns = list(df.columns)
            for map_idx, map_name in enumerate(COLUMN_MAPPING[task]):
                if map_name == "id" and task.startswith("tabular"):
                    st.selectbox(f"Map `{map_name}` to:", columns + [""], index=map_idx, key=f"map_{map_name}")
                else:
                    st.selectbox(f"Map `{map_name}` to:", columns, index=map_idx, key=f"map_{map_name}")

        st.markdown("###### Model choice")
        if task.startswith("tabular"):
            model_choice = "AutoTrain"
        elif task == "dreambooth":
            model_choice = "HuggingFace Hub"
        else:
            model_choice_label = ["AutoTrain", "HuggingFace Hub"]
            model_choice = st.selectbox("Model Choice", model_choice_label, label_visibility="collapsed")

        hub_model = None
        if model_choice == "HuggingFace Hub":
            default_hub_model = "bert-base-uncased"
            if task == "dreambooth":
                default_hub_model = "stabilityai/stable-diffusion-2-1-base"
            hub_model = st.text_input("Model name", default_hub_model)

        st.sidebar.markdown("### Parameters")
        params = Params(task=task, training_type="autotrain" if model_choice == "AutoTrain" else "hub_model")
        params = params.get()

        if model_choice == "AutoTrain":
            st.sidebar.markdown("Hyperparameters are selected automagically for AutoTrain models")

        for key, value in params.items():
            if value.STREAMLIT_INPUT == "selectbox":
                st.sidebar.selectbox(value.PRETTY_NAME, value.CHOICES, 0, key=f"params__{key}")
            elif value.STREAMLIT_INPUT == "number_input":
                try:
                    step = value.STEP
                except AttributeError:
                    step = None
                try:
                    _format = value.FORMAT
                except AttributeError:
                    _format = None
                st.sidebar.number_input(
                    value.PRETTY_NAME,
                    value.MIN_VALUE,
                    value.MAX_VALUE,
                    value.DEFAULT,
                    step=step,
                    format=_format,
                    key=f"params__{key}",
                )
            elif value.STREAMLIT_INPUT == "slider":
                st.sidebar.slider(
                    value.PRETTY_NAME,
                    value.MIN_VALUE,
                    value.MAX_VALUE,
                    value.DEFAULT,
                    key=f"params__{key}",
                )
        if model_choice == "AutoTrain":
            st.session_state.jobs = []
            st.session_state.jobs.append(
                {k[len("params__") :]: v for k, v in st.session_state.items() if k.startswith("params__")}
            )
        else:
            add_job = st.sidebar.button("Add job")
            delete_all_jobs = st.sidebar.button("Delete all jobs")

            if add_job:
                if "jobs" not in st.session_state:
                    st.session_state.jobs = []
                st.session_state.jobs.append(
                    {k[len("params__") :]: v for k, v in st.session_state.items() if k.startswith("params__")}
                )

            if delete_all_jobs:
                st.session_state.jobs = []

    # show the grid with parameters for hub_model training
    selected_rows = []
    if "jobs" in st.session_state and "model_choice" in locals():
        if model_choice != "AutoTrain":
            if len(st.session_state.jobs) == 1 and "num_models" in st.session_state.jobs[0]:
                st.session_state.jobs = []
            if len(st.session_state.jobs) > 0:
                ag_resp = create_grid(st.session_state.jobs)
                ag_resp_sel = ag_resp["selected_rows"]
                if ag_resp_sel:
                    selected_rows = [
                        int(ag_resp_sel[i]["_selectedRowNodeInfo"]["nodeId"]) for i in range(len(ag_resp_sel))
                    ]
                    logger.info(selected_rows)
                st.markdown("<p>Only selected jobs will be used for training.</p>", unsafe_allow_html=True)

    # create project
    # step1: process dataset
    # step2: create project using AutoTrain API
    # step3: estimate costs
    # step4: start training if user confirms
    if not verify_project_name(project_name=project_name, username=autotrain_username, user_token=user_token):
        return
    if model_choice != "AutoTrain":
        if "jobs" not in st.session_state:
            st.error("Please add at least one job")
            return
        if len(st.session_state.jobs) == 0:
            st.error("Please add at least one job")
            return
        if len(selected_rows) == 0:
            st.error("Please select at least one job")
            return
    logger.info(st.session_state)
    try:
        if task == "dreambooth":
            dset = AutoTrainDreamboothDataset(
                num_concepts=number_of_concepts,
                concept_images=[
                    st.session_state[f"dreambooth_concept_images_{i + 1}"] for i in range(number_of_concepts)
                ],
                concept_names=[
                    st.session_state[f"dreambooth_concept_name_{i + 1}"] for i in range(number_of_concepts)
                ],
                token=user_token,
                project_name=project_name,
                username=autotrain_username,
            )
        else:
            dset = AutoTrainDataset(
                train_data=training_data,
                task=task,
                token=user_token,
                project_name=project_name,
                username=autotrain_username,
                column_mapping={map_name: st.session_state[f"map_{map_name}"] for map_name in COLUMN_MAPPING[task]},
                valid_data=validation_data,
                percent_valid=None,  # TODO: add to UI
            )

        logger.info(f"Number of samples: {dset.num_samples}")

        estimated_cost = get_project_cost(
            username=autotrain_username,
            token=user_token,
            task=task,
            num_samples=dset.num_samples,
            num_models=len(selected_rows) if model_choice != "AutoTrain" else st.session_state.jobs[0]["num_models"],
        )
        st.write(f"Estimated cost: {estimated_cost} USD")
    except Exception as e:
        logger.error(e)
        st.error("Error while creating project. Please check your inputs and try again.")
        return

    # create project button
    create_project_button = st.button("Create Project")

    if create_project_button:
        with st.spinner("Munging data and uploading to 🤗 Hub..."):
            dset.prepare()

        project = Project(
            dataset=dset,
            hub_model=hub_model,
            job_params=get_job_params(st.session_state.jobs, selected_rows, task, model_choice),
        )
        with st.spinner("Creating project..."):
            project_id = project.create()
        with st.spinner("Approving project for training..."):
            project.approve(project_id)

        st.success(
            "Project created successfully. Monitor progess on the [dashboard](https://ui.autotrain.huggingface.co/projects)."
        )


if __name__ == "__main__":
    args = parse_args()
    st.session_state.task = args.task
    app()
